/****************************************************************************************
 * Obsługa USART1 bez uzycia przerwan                                                   *
 * Zadanie Z002 do kursu                                                                *
 * USART1 w nowszych wersjach stm32f429I-Discovery podpięty jest do zintegrowanego      *
 * programatora. W systemie po podpięciu płytki,powinien być widziany dodatkowy         *
 * port szeregowy. Programem terminalowym np. PUTTY możemy się podpiąć do wirtualnego   *
 * portu COM (/dev/ttyACM[0-9] w linuksie) i będziemy mogli komunikowaæ się z naszym    *
 * przykładem                                                                           *
 * autor: sq8mvy Paweł                                                                  *
 ***************************************************************************************/

#include "stm32f4xx.h"

#include "usart1.h"             //Dołączamy plik nagłówkowy z obsługą naszego USART1
                                //W pliku usart1.h umieszczone są DEKLARACJE funkcji
                                //które mają być dostępne publicznie
                                //W pliku usart1.c natomiast są Definicje funkcji, czyli
                                //ciało funkcji...

#define ile_tickow  2           //liczba przerwań SysTick w ciągu sekundy


volatile char dana;

void powitanie(void)            //Utworzymy funkcję "powitanie", będziemy w dalszej części
                                //z niej korzystać
{
  // Funkcja powitanie będzie wysyłać coś na port szeregowy....
    usart1_puts("\033[2J\n\r");                                   //kod ANSI \033[2J - czyszczenie ekranu terminala
    usart1_puts("Zadanie Z002 do kursu...\n\r");                  // \n - przejscie do nastepnej linii
    usart1_puts("Obsluga portu szeregowego\n\r");                 // \r - przejscie do poczatku linii
    usart1_puts("bez uzycia przerwan.\n\r\n\r");
    usart1_puts("Konfiguracja parametrow portu szeregowego\n\r");
    usart1_puts("Predkosc:          115200 bit/s\n\r");
    usart1_puts("Ramka:             8 bit\n\r");
    usart1_puts("Bit parzystosci:   brak\n\r");
    usart1_puts("Bity stopu:        1 bit\n\r");
    usart1_puts("Echo programowe:  wlaczone\n\r\n");

}

int main(void)
{

    SystemCoreClockUpdate();            //Aktualizujemy zmienną globalną SystemCoreClock
                                        //przechowującą częstotliwość taktowania

    usart1_Init(115200);                //Inicjalizujemy port szeregowy USART1
                                        //Prędkość 115200 bit/s, ramka 8 bitów, bez bitu parzystości, 1 bit stopu

    powitanie();                        //Wywołamy funkcję "powitanie", która coś wyśle na port szeregowy



// Wlaczenie taktowania  dla GPIOG. Na pinach PG13 i PG14 są podpięte doidy LED. Zielona i Czerwona

    RCC->AHB1ENR |= RCC_AHB1ENR_GPIOGEN;



// Pin 13 oraz 14 portu GPIOG jako wyjście dla LED
// Piny ustawione jako wyjsciowe domyślnie są Push-Pull
// więc nie ma potrzeby ustawiać pinow 13 oraz 14 w rejestrze OTYPER dla GPIOG

    GPIOG->MODER |= GPIO_MODER_MODER13_0 | GPIO_MODER_MODER14_0;



// Włączymy jedną diodę, aby potem w przerwaniu SysTicka mrugać naprzemiennie
// doidami...

    GPIOG->ODR |= GPIO_ODR_ODR_13;



//Skonfigurujemy SysTicka, aby diodki radośnie mrugały do nas....

    if(SysTick_Config(SystemCoreClock/ile_tickow))
    {
        SysTick_Config((SystemCoreClock/8)/ile_tickow);
        SysTick->CTRL = SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;
    }




// Tu się zaczyna pętla nieskończona.......
    while(1)
    {
      usart1_puts("Teraz mozesz cos napisac, konczac klawiszem ENTER\r\n->");
      usart1_gets();
      usart1_puts("Twoj wpisany string: \033[1m");
      usart1_puts(usart_bufor_rx);
      usart1_puts("\033[0m\r\n\n");
    }


}

// Przerwanie SysTicka
// W przerwaniu tym mrugamy naprzemiennie diodami LED, zieloną oraz czerwoną

void SysTick_Handler(void)
{
    GPIOG->ODR ^=GPIO_ODR_ODR_13 | GPIO_ODR_ODR_14;             // Instrukcja ta działa następująco..
                                                                // Odczytuje zawartość rejestru ODR portu GPIOG
                                                                // zamienia na przeciwne wartości bitów 13 oraz 14
                                                                // i tak zmodyfikowaną wartość zapisuje spowrotem
                                                                // do rejestru ODR portu GPIOG
                                                                // ODR - jest to 32 bitowy rejestr wyjściowy portu,
                                                                // Jednak tylko 16 najmłodszych bitów tego rejestru
                                                                // niosą informacje, która pojawi się na pinach 0..15
                                                                // danego portu
                                                                // Odczytując rejestr ODR nie odczytujemy rzeczywistego
                                                                // stanu pinów, lecz wartość którą tam zapisaliœmy
                                                                // Rzeczywisty stan pinów uzyskamy odczytując rejest IDR
                                                                // danego portu
}
