/****************************************************************************************
 * Obsługa USART1 bez użycia przerwań                                                   *
 * Zadanie Z002 do kursu                                                                *
 * USART1 w nowszych wersjach stm32f429I-Discovery podpięty jest do zintegrowanego      *
 * programatora. W systemie po podpięciu płytki,powinien być widziany dodatkowy         *
 * port szeregowy. Programem terminalowym np. PUTTY możemy się podpiąć do wirtualnego   *
 * portu COM (/dev/ttyACM[0-9] w linuksie) i będziemy mogli komunikować się z naszym    *
 * przykładem                                                                           *
 * autor: sq8mvy Paweł                                                                  *
 ***************************************************************************************/
#include "stm32f4xx.h"

#include "usart1.h"             //Dołączamy plik nagłówkowy z obsługą naszego USART1
                                //W pliku usart1.h umieszczone są między innymi DEKLARACJE funkcji
                                //które mają być dostępne publicznie
                                //W pliku usart1.c natomiast są Definicje funkcji, czyli
                                //ciało funkcji...


// Procedura inicjalizacyjna. Konfigurujemy w niej port szeregowy USART1
void  usart1_Init(uint32_t bautrate)
{

//Włączamy taktowanie portu GPIOA. Na pinach PA9 i PA10 tego portu jest dostępny USART1 jako funkcja alternatywna
    RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;


// Wlaczenie taktowania dla USART1
    RCC->APB2ENR |= RCC_APB2ENR_USART1EN;


// UART1_TX - PA9, UART1_RX - PA10 jako funkcja AF
    GPIOA->MODER |= GPIO_MODER_MODER9_1 | GPIO_MODER_MODER10_1;


//Alternatywną funkcją dla PA9 oraz PA10 portu GPIOA jest USART1..3
    GPIOA->AFR[1] |= (uint32_t)0x770;


//Ustawiamy Pull-Up-a dla nóżki RX-a, aby nie wisiał w powietrzu, jak nie będzie do niczego podpięta
//Nóżka PA9 - TX jako wyjściowa domyślie jest PushPull, więc nie ma potrzeby jej konfigurować
    GPIOA->PUPDR |= GPIO_PUPDR_PUPDR10_1;


// Ustawiamy predkosc dla USART1 z uwzglednieniem preskalera dla szyny APB2
// Do której jest podpiety USART1

    uint32_t apb2_div;

    apb2_div = RCC->CFGR & 0xE000;

    switch(apb2_div)
    {
        case RCC_CFGR_PPRE2_DIV2:
            apb2_div = 2;
        break;

        case RCC_CFGR_PPRE2_DIV4:
            apb2_div = 4;
        break;

        case RCC_CFGR_PPRE2_DIV8:
            apb2_div = 8;
        break;

        case RCC_CFGR_PPRE2_DIV16:
            apb2_div = 16;
        break;

        default:
            apb2_div = 1;
    }

    USART1->BRR = (SystemCoreClock/apb2_div)/bautrate;


// Włączamy peryferial UART1, TX oraz RX

    USART1->CR1 = USART_CR1_UE | USART_CR1_TE | USART_CR1_RE;
}



//Funkcja wysyłająca 1 znak przez port szeregowy USART1

void usart1_putc(char znak)
{
    while(!(USART1->SR & USART_SR_TXE));        //Sprawdzamy, czy bufor nadawczy jest już pusty
                                                //Jeżeli jeszcze nie, to czekamy w pętli, aż bufor
                                                //nadawczy będzie pusty

    USART1->DR =znak;                           //Jeżeli bufor nadawczy jest już pusty, wysyłamy znak,
                                                //który przekazany został do tej funkcji


}



//Funkcja wysyłająca string przez port szeregowy...
//Do funkcji nie jest przekazywany cały string, tylko adres komórki pamięci pierwszego bajtu stringa

void usart1_puts(const char * tekst)
{
    while (*tekst)                              //każdy string w języku C zakończony jest bajtem o wartości 0
                                                //odczytujemy znak spod adresu w pamięci, jeśli jest to bajt o wartości 0
                                                //pętla się zakończy
                                                //Jeśli jest to wartość różna od 0,
                                                //zostanie wysłana przez port szeregowy
    {
        usart1_putc(*tekst++);                  // wysyłamy 1 znak, na który wskazuje wskaźnik, przez port szeregowy
                                                // następnie zwiększamy adres wskazujący na znak o 1
    }
}



//Funkcja odczytu odebranego znaku z portu szeregowego

char usart1_getc(void)
{
    if (USART1->SR & USART_SR_RXNE)               //Sprawdzamy, czy w buforze odbiorczym USART
    {                                             //jest do odczytania odebrana dana (bit RXNE w rejestrze Statusu)
      return (char)USART1->DR;                    //jeœli jest, odczytujemy z bufora i zwracamy do funkcji wywołującej
    }
    return 0;                                     // Jeśli nie ma, zwracamy 0
}



//Funkcja do odczytu stringa przez port szeregowy. Kończy działanie po naciśnięciu klawisza ENTER

void usart1_gets(void)
{
  uint8_t tmp = 0;                               //definiujemy tymczasową zmienną, potrzebną
                                                 //do iteracji tablicy  bufora usart_bufor_rx[]
  char znak;                                     //tymczasowa zmienna na odebrany znak

  while(1)
  {
    while(!(znak=usart1_getc()));               //oczekuje w nieskończonej pętli na jakiś odebrany znak
    if(znak =='\r' || tmp==bufor_size_rx-1)     //Jeśli odebrany znak to Enter, lub zmienna tmp przyjmie
                                                //wartość max wielkości bufora na dane to.......

    {
      usart_bufor_rx[tmp]='\0';             //Wstawiamy do bufora wartość 0, mówiącą, że to koniec stringa
      usart1_puts("\n\r");                  //Na terminal wysyłamy powrót karetki do początku nowej linii
                                            //Ustawienie karetki na początku natępnej linii nie jest konieczne,
                                            //bo prawdopodobnie i tak w programie będziemy to robić......
                                            //Ja to zastosowałem, bo ładniej wygląda jak po naciśnięciu klawisza ENTER
                                            //Karetka ustawia się prawidłowo...
      return;                               //wychodzimy z funkcji

    } else                                //W przeciwnym wypadku.......
    {
      USART1->DR = znak;                  //Wysyłamy do terminala to co do nas dotarło portem szeregowym.
                                          //Jest to tak zwane echo.
                                          //UWAGA. Jeśli w programie terminalowym będzie włączone echo lokalne,
                                          //to będą się wyświetlały po dwa jednakowe znaki.
                                          //Jeden ten, który jest odsyłany przez tą funkcję, drugi
                                          //będzie wstawiany przez terminal jako echo lokalne.
      usart_bufor_rx[tmp] = znak;         //Wpisujemy do bufora odebrany znak
      tmp++;                              //zwiększamy o 1 zmienną iteracyją bufora wejściowego
    }
  }
}
